@@ -186,13 +186,11 @@ int snd_timer_continue(snd_timer_t *handle)
 
 ssize_t snd_timer_read(snd_timer_t *handle, void *buffer, size_t size)
 {
-	snd_timer_t *tmr;
 	ssize_t result;
 
-	tmr = handle;
-	if (!tmr || (!buffer && size > 0) || size < 0)
+	if (handle == NULL || buffer == NULL || size == 0)
 		return -EINVAL;
-	result = read(tmr->fd, buffer, size);
+	result = read(handle->fd, buffer, size);
 	if (result < 0)
 		return -errno;
 	return result;
